# $Id: ndiary-lib.rb,v 1.23 2002/06/25 12:30:16 not Exp $

NDIARY_VERSION = 'nDiary version 0.9.3.beta18'

REQUIRED_RUBY_VER = '1.6'

RUBY_INFO = "ruby #{RUBY_VERSION} (#{RUBY_RELEASE_DATE}) [#{RUBY_PLATFORM}]"

require 'lib/config'
require 'lib/string'
require 'lib/dateformat'
require 'lib/find'


module DiaryModule

  attr_reader :logDirectory
  attr_reader :outputDirectory
  attr_accessor :userPluginDirectory

  #typo_fix
  def typo_fix
    @diarys = @diaries
    @allDiarys = @allDiaries
  end
  
  def isXHTML
    if @isxhtml.nil? then
      unless @isxhtml = @config['OUTPUT_XHTML'] then
        begin
          open(@logDirectory + @skelton){ |f|
            if /<!DOCTYPE [^>]+XHTML/im =~ f.gets('</') then
              @isxhtml = true
            else
              @isxhtml = false
            end
          }
        rescue
          # $stderr.puts "skelton file '#{@logDirectory + @skelton}' not found"
          @isxhtml = false
        end
      end
    end
    @isxhtml
  end

  def tag_br(str = '')
    str = ' ' + str unless str.empty?
    isXHTML ? "<br#{str} />" : "<br#{str}>"
  end

  def tag_hr(str = '')
    str = ' ' + str unless str.empty?
    isXHTML ? "<hr#{str} />" : "<hr#{str}>"
  end

  def setlogDirectory(dir)
#    dir = File::expand_path(dir)
    dir << '/' unless dir[-1,1] == '/'
    unless File::directory?(dir) then
      $stderr.puts "\n** error: '#{dir}' not exist.\n"
      exit
    end
    @logDirectory = dir
    if @logDirectory == @outputDirectory then
      $stderr.puts "\n** error: LOG_DIRECTORY and OUTPUT_DIRECTORY should not be the same directory.\n"
      exit
    end      
  end

  def setoutputDirectory(dir)
#    dir = File::expand_path(dir)
    dir << '/' unless dir[-1,1] == '/'
    unless File::directory?(dir) then
      $stderr.puts "\n** error: '#{dir}' not exist.\n"
      exit
    end
    @outputDirectory = dir
    if @logDirectory == @outputDirectory then
      $stderr.puts "\n** error: LOG_DIRECTORY and OUTPUT_DIRECTORY should not be the same directory.\n"
      exit
    end      
  end

  def setmonthlyFilename(str)
    return if str.to_s.empty?
    if str.index('%0m') and str.index('%Y') then
      @monthlyFilename = str
    else
      $stderr.puts "\n** error: MONTHLY_FILENAME wrong.\n"
      exit
    end
  end

  def logDirectory=(dir);         setlogDirectory(dir);         end
  def outputDirectory=(dir);      setoutputDirectory(dir);      end
  def monthlyFilename=(filename); setmonthlyFilename(filename); end
  
  # convet date to monthly filename
  def date2monthlyfilename(date)
    yyyy = date[0..3]
    mm   = date[4..5]
    dd   = date[6..7]

    @monthlyFilename.gsub('%Y', yyyy).gsub('%0m', mm).gsub(/^(.+)(\.[^\/]+)$/,"\\1#{@monthPartNames[dd.to_i]}\\2")
  end

  def date2fileanchor(date)
    (isXHTML ? 'd' : '' ) + date[6..7]
  end

  # convert date to relative link string
  def date2monthlyfilelink(date, anchor = nil)
    filename = date2monthlyfilename(date)
    if anchor then
      filename << '#'
      filename << date2fileanchor(date)
    end
    '../' * @filename.scan(%r"[^/]+/").size + filename
  end

  # Load Plug-in
  def loadplugin(line)
    #typo_fix
    typo_fix
    
    line.gsub!(/<!--plug=(.+)-->/i){
      argv = []
      argv = $1.gsub('&gt;','>').gsub('&lt;','<').split(/\?/s)
      plugname = argv.shift
      plugname = plugname + '.rb'

      opt = {}
      argv.each{ |str|
        str = str.gsub('&question;','?').gsub('&amp;','&')
        key, val = str.split(/=/, 2)
        opt[key.downcase] = val
      }

      if $DEBUG then
        if @userPluginDirectory
          begin
            load @userPluginDirectory + '/' + plugname
          rescue LoadError
            load 'plugin/' + plugname
          end
        else
          load 'plugin/' + plugname
        end
        plug(opt)
      else
        begin
          if @userPluginDirectory
            begin
              load @userPluginDirectory + '/' + plugname
            rescue LoadError
              load 'plugin/' + plugname
              true
            end
          else
            load 'plugin/' + plugname
          end
          plug(opt)
        rescue
          $stderr.puts "\n** Plug-in error: #{plugname}\n"
          "<!--plug-in error: #{plugname}-->"
        end
      end
    }
  end

end


require 'lib/topic'

class Filter

  attr_reader :diary

  def initialize(filterlist = [], ufdir = nil)
    @filterlist = filterlist
    @filterlist = [] if @filterlist.nil?
    @filterlist.delete_if{ |filter|
      begin
        if ufdir
          begin
            require ufdir + '/' + filter + '.rb'
            puts 'load: ' + ufdir + '/' + filter + '.rb' if $DEBUG
          rescue LoadError
            require 'filter/' + filter + '.rb'
            puts 'load: filter/' + filter + '.rb' if $DEBUG
          end
        else
          require 'filter/' + filter + '.rb'
          puts 'load: filter/' + filter + '.rb' if $DEBUG
        end
        false
      rescue LoadError
        $stderr.puts "\n** error: Filter '#{filter}.rb' not found.\n"
        true
      end
    }
  end

  def parentdiary=(diary)
    @diary = diary
  end

  def filter(str, type)
    ret = false
    @filterlist.each{ |filter|
      begin
        ret = true if send(filter, str, type) == :THROUGH
      rescue NameError
        $stderr.puts "\n** Filter error: '#{filter}'.\n"
        @filterlist.delete(filter)
      end
    }
    return ret
  end

end


class Diary
  require 'cgi'
  require 'ftools'
  include DiaryModule
  
  attr_accessor :latestFilename
  attr_reader   :monthlyFilename
  attr_accessor :monthPartNames
  attr_accessor :showVersion
  attr_accessor :format
  attr_accessor :months
  attr_accessor :date
  attr_accessor :header
  attr_accessor :topicChar
  attr_accessor :outputDirectory
  attr_accessor :tag_date
  attr_accessor :tag_topic
  attr_accessor :paragraphAnchor 
  attr_accessor :anchorEachTopic
  attr_accessor :lf2br
  attr_accessor :getDateByTimestamp
  attr_accessor :splithtml
  attr_accessor :skelton
  attr_accessor :outputKcode
  attr_reader   :config
  attr_reader   :filter

  attr_reader :months
  attr_reader :diaries
  attr_reader :allDiaries
  attr_reader :filename

  #typo fix
  attr_reader :diarys
  attr_reader :allDiarys
  alias_method("allDiarys", "allDiaries")
  alias_method("diarys", "diaries")

  def initialize
    @diaries         = []
    @showVersion     = true
    @logDirectory    = ''
    @outputDirectory = ''
    @allDiaries      = {}
    @months          = []
    @format          = '%b.%d,%Y (%a)'
    @header          = ''
    @topicChar       = ''
    @lf2br           = true
    @outputKcode     = 1
    @splithtml       = ''
    @now             = Time::now
    @filter          = nil
    @latestFilename  = 'index.html'
    @monthlyFilename = '%Y%0m.html'
    @filename        = ''

  end

  def config=(conf)
    @config = conf

    @showVersion = @config['SHOW_VERSION'] if @config.key?('SHOW_VERSION')
    @latestFilename = @config['LATEST_FILENAME'] if @config.key?('LATEST_FILENAME')
    @outputKcode = @config['OUTPUT_KCODE'] if @config.key?('OUTPUT_KCODE')
    @topicChar = @config['TOPIC_CHAR'] if  @config.key?('TOPIC_CHAR')
    @paragraphAnchor = @config['PARAGRAPH_ANCHOR'] if @config.key?('PARAGRAPH_ANCHOR')
    @anchorEachTopic = @config['ANCHOR_EACH_TOPIC'] if @config.key?('ANCHOR_EACH_TOPIC')
    @lf2br = @config['LF2BR'] if @config.key?('LF2BR')
    @topic_sentence_link = @config['TOPIC_SENTENCE_LINK'] if @config.key?('TOPIC_SENTENCE_LINK')
    @userPluginDirectory = @config['USER_PLUGIN_DIRECTORY']

    setlogDirectory(@config['LOG_DIRECTORY'])
    setoutputDirectory(@config['OUTPUT_DIRECTORY'])
    setmonthlyFilename(@config['MONTHLY_FILENAME'])
  end

  def tag_date
    if @tag_date.to_s.empty? then
      'h4'
    else
      @tag_date
    end
  end

  def tag_topic
    if @tag_topic.to_s.empty? then
      'h5'
    else
      @tag_topic
    end
  end

  def filter=(f)
    @filter = f
    @filter.parentdiary = self
  end

  # search diary file
  def getDiaries(diary = nil)
    if diary.kind_of?(Diary) then
      @allDiaries.update(diary.allDiaries)
      @months = diary.months
    else
      Dir::find(@logDirectory){ |file|
        next unless /\/(\d{8})(\.diary|\.dia)$/i =~ file
        date = File::basename(file, $2)
        unless @allDiaries.key?(date) then
          @allDiaries[date] = file
        else
          $stderr.puts "\n** error: #{date}'s diary duplicated\n"
          exit
        end
      }

      @allDiaries.keys.each{ |date|
        @months << date[0..5]
      }
      for year in "1990"..Time::now.strftime("%Y")
        for month in '01'..'12'
          if File::exist?(@outputDirectory + @monthlyFilename.gsub(/%Y/, year).gsub(/%0m/, month)) then
            @months << year + month
          end
        end
      end
      @months = @months.uniq.sort
    end
  end

  alias getDiarys getDiaries

  def makeFootNote(dstr)
    str = dstr.dup
    str.gsub!(/\([\*]([^\)]+)\)|i[\*]([^j]+)j|\(\(-(.+?)-\)\)/){
      @footnotes << $+
      name = "#{@date}_#{@footnotes.size}"
      "<sup><small>(<a href=\"#f#{name}\" #{isXHTML ? 'id' : 'name'}=\"b#{name}\">*#{@footnotes.size}</a>\)</small></sup>"
    }
    str.gsub!(/\(\\\*/,"\(\*")
    return str
  end

  private

  def putfootnote(body)
    return body if @footnotes.empty?
    cnt = 0
    footnote = "\t<div class=\"footnote\">" + tag_hr('class="footnote"') +"\n"
    @footnotes.each{ |str|
      cnt += 1
      name = "#{@date}_#{cnt}"
      footnote << "\t\t<a #{isXHTML ? 'id' : 'name'}=\"f#{name}\" "
      footnote << "href=\"#b#{name}\">\*#{cnt}</a>: #{str}#{tag_br}\n"
    }
    footnote << "\t</div>\n"
    body + footnote
  end

  def makeParagraph(str)
    return str if @filter.filter(str, :PRE)
    
    if str[0,3] == "<<\n" then
      return str if @filter.filter(str, :HEREDOC)
      str = str[3...str.size]
      str.gsub!("\n","\n\t")
      str = "\t#{str}\n\n"
      @filter.filter(str, :AFTER_HEREDOC)
      return str

    elsif /\ACode:(?: (-.*))?\n/i =~ str then
      str = $'
      param = $1
      return str if @filter.filter(str, :CODE)
      t = /t(\d+)/i =~ param ? $1.to_i : 4
      str = CGI::escapeHTML(str) if /q/i =~ param
      str = str.expandtabs(t)
      str = "\t<div class=\"code\"><pre><tt><code>\n#{str}</code></tt></pre></div>\n\n"
      @filter.filter(str, :AFTER_CODE)
      return str

    elsif /\AQuote:(?: (-.*))?\n/i =~ str then
      str = $'
      param = $1
      return str if @filter.filter(str, :QUOTE)
      t = /t(\d+)/i =~ param ? $1.to_i : 4
      str = CGI::escapeHTML(str) if /q/i =~ param
      if /l[^-]/i =~ param or (@lf2br != false and /l-/i !~ param) then
        str.gsub!("\n", tag_br + "\n")
      end
      str = str.expandtabs(t)
      str = "\t<blockquote class=\"quote\">\n#{str}</blockquote>\n\n"
      @filter.filter(str, :AFTER_QUOTE)
      return str

    elsif /^[^\t]/ !~ str then
      return str if @filter.filter(str, :BLOCKQUOTE)
      str.delete!("\t")
      str.gsub!("\n", tag_br + "\n\t\t") unless @lf2br == false
      str = makeFootNote(str)
      str = "\t<blockquote><p>\n\t\t#{str}\n\t</p></blockquote>\n\n"
      @filter.filter(str, :AFTER_BLOCKQUOTE)
      return str

    elsif /^\t/ =~ str then
      return str if @filter.filter(str, :DL)
      str.gsub!(/^/,"\t\t<dt>")
      str.gsub!(/^\t\t<dt>\t/,"\t\t\t<dd>")
      str.gsub!(/^(\t\t<dt>.+)$/,"\\1</dt>")
      str.gsub!(/^(\t\t\t<dd>.+)$/,"\\1</dd>")
      str = makeFootNote(str)
      str = "\t<dl>\n#{str}\n\t</dl>\n\n"
      @filter.filter(str, :AFTER_DL)
      return str

    elsif @topicChar.index((th, ts = str.split(//,2))[0]) and (/\n/ !~ str) then
      return str if @filter.filter(ts, :TOPIC)
      @cntTopic += 1
      @cntParagraph = 0 if @anchorEachTopic == true
      if self.kind_of?(PastDiary) then
        @topics[@dd] = [] unless @topics.key?(@dd)
        @topics[@dd] << th + ts
      end
 
      name = ''
      if self.kind_of?(PastDiary)
        name = (isXHTML ? "id" : "name") + "=\"#{date2fileanchor(@date)}_t#{@cntTopic}\" "
      end

      if @topic_sentence_link then
        links = []
        ts.gsub!(/<a href="[^\"]+">([^<]+)<\/a>/){
          links << $&
          $1
        }
        link = ''
        if links.size.nonzero? then
          link = " <small class=\"topic_ref\">(QURL: #{links.join(' / ')})</small>"
        end

        str = "\t<#{tag_topic} class=\"topic\"><a #{name}href=\"#{date2monthlyfilelink(@date, true)}_t#{@cntTopic}\" class=\"topic\">#{th}#{ts}</a>#{link}</#{tag_topic}>\n\n"
      else
        str = "\t<#{tag_topic}><a #{name}href=\"#{date2monthlyfilelink(@date, true)}_t#{@cntTopic}\" class=\"topic\">#{th}</a>#{ts}</#{tag_topic}>\n\n"
      end

      @filter.filter(str, :AFTER_TOPIC)
      return str

    elsif /\A[\+E]/ =~ str then
      return str if @filter.filter(str, :UL)
      str.gsub!(/\n([^\+E].+)$/,"#{tag_br}\\1") unless @lf2br == false
      str.gsub!(/(?:\n|^)[\+E](.+)(?!^[\+E])/,"\n\t\t<li>\\1</li>")
      str = makeFootNote(str)
      str = "\t<ul>#{str}\n\t</ul>\n\n"
      @filter.filter(str, :AFTER_UL)
      return str

    elsif !str.strip.empty? then
      return str if @filter.filter(str, :P)
      @cntParagraph += 1
      str.gsub!("\n", tag_br + "\n\t") unless @lf2br == false
      str = makeFootNote(str)
      if @paragraphAnchor then
        anchor = ''
        if @anchorEachTopic then
          anchor = "_t#{@cntTopic}_#{@cntParagraph}"
        else
          anchor = "_#{@cntParagraph}"
        end
        name = ''
        if self.kind_of?(PastDiary)
          name = (isXHTML ? "id" : "name") + "=\"#{date2fileanchor(@date)}#{anchor}\" "
        end

        str = "\t<p><a #{name}href=\"#{date2monthlyfilelink(@date, true)}#{anchor}\">#{@header}</a>\n\t#{str}</p>\n\n"
      else
        str = "\t<p>#{str}</p>\n\n"
      end
      @filter.filter(str, :AFTER_P)
      return str

    else 
      return ''
    end
  end

  def tohtml(date)
    body = ''
    @cntParagraph = @cntTopic = 0
    @footnotes = []
    filename = @allDiaries[date]
    
    @date = date
    @yyyy = @date[0..3]
    @mm   = @date[4..5]
    @dd   = @date[6..7]

    objDate = Time::local(@yyyy, @mm, @dd)

    # Read Diary File
    text = ''
    open(filename){ |file| text = file.read.gsub(/\r\n/,"\n") }
    text = text.tosjis

    bodys = [[]]

    # Filter :ONEDAY
    if @filter.filter(text, :ONEDAY) then
      body << text
    else
      text.each("\n\n"){ |line|
        line.sub!(/^\n$/,"")
        line.chomp!
        line.chomp!
        line.delete!("\C-z")
        next if line.empty?
        str = makeParagraph(line)
        bodys[@cntTopic] = [] if bodys[@cntTopic].nil?
        bodys[@cntTopic] << str
      }
    end

    t0 = bodys.shift
    if t0.size.nonzero? then
      body << "\t<div class=\"topic_body\">\n\n#{t0.join}\t</div>\n\n"
    end
    
    bodys.reverse! if @topicSortReverse
    bodys.each{ |block|
      body << block[0]
      body << "\t<div class=\"topic_body\">\n\n"
      body << block[1..-1].join
      body << "\t</div>\n\n"
    }
    
    body = putfootnote(body)

    body = "<div class=\"day\">\n\n" + body + "</div>\n\n"

    name = ''
    if self.kind_of?(PastDiary) then
      name = (isXHTML ? "id" : "name") + "=\"#{date2fileanchor(@date)}\" "
    end
    date_anchor = objDate.strftime2("<#{tag_date} class=\"date\"><a #{name}href=\"#{date2monthlyfilelink(@date, true)}\" class=\"date\">#{@format}</a></#{tag_date}>\n")
    @filter.filter(date_anchor, :DATE_ANCHOR)
    body = date_anchor + body

    splithtml = objDate.strftime2(@splithtml).gsub(/\"/, '\"')
    splithtml = eval('"' + splithtml + '"')
    body << splithtml + "\n\n" unless @splithtml.empty?

    
    # Filter :AFTER_ONEDAY
    @filter.filter(body, :AFTER_ONEDAY)

    return body
  end

  def writeHTML
#    filename = File::expand_path(@filename, @outputDirectory)
    filename = @outputDirectory + @filename
    @outputKcode = 1 if (@outputKcode < 1 or @outputKcode > 3)
    begin

      body = ''
      @diaries.each{ |date|
        body << tohtml(date).kconv(@outputKcode)
      }

      File::makedirs(File::dirname(filename))

      out = open(filename, "w")

      infile = open(@logDirectory + @skelton)
      infile.each { |line|

        if line.gsub!(/<!--body-->/i, '') then

          if @showVersion then
            out.print "\n<div class=\"version\">Generated by ",
                      "<a href=\"http://www14.cds.ne.jp/~not/product/ndiary/\">",
                      NDIARY_VERSION, "</a></div>\n#{tag_hr}\n"
          else
            out.print "\n<!--Generated by #{NDIARY_VERSION}-->\n"
          end
          
          lm = @now.dup.gmtime
          out.puts lm.strftime("<!--Last-Modified :%a, %d %b %Y %H:%M:%S GMT-->")

          out.print body
          
          out.print "<!--#{RUBY_INFO}-->\n\n"

        end

        loadplugin(line)

        out.print line.kconv(@outputKcode)
      }
      infile.close

      out.close

      print "make '#{filename}'\n"

    rescue
      $stderr.puts "\n** error: skelton file '#{@logDirectory}#{@skelton}' not found.\n"
      exit
    end
  end

end


class LastDiary < Diary
  attr_accessor :maxDiaries
  attr_accessor :topicSortReverse

  alias maxDiarys maxDiaries
  
  def initialize
    super()
    @maxDiaries = 5
    @topicSortReverse = false
  end

  alias maxDiarys maxDiaries
  
  def config=(conf)
    super
    @maxDiaries = @config['MAX_DIARY'] if @config.key?('MAX_DIARY')
    @skelton = @config['SKELTON_LATEST'] if @config.key?('SKELTON_LATEST')
    @splithtml = @config['SPLIT_LATEST'] if @config.key?('SPLIT_LATEST')
    @format = @config['DATEFORMAT_LATEST'] if @config.key?('DATEFORMAT_LATEST')
    @header = @config['HEADER_LATEST'] if @config.key?('HEADER_LATEST')
    @tag_date = @config['TAG_DATE_LATEST'] if @config.key?('TAG_DATE_LATEST')
    @tag_topic = @config['TAG_TOPIC_LATEST'] if @config.key?('TAG_TOPIC_LATEST')
    @topicSortReverse = @config['TOPIC_SORT_REVERSE']
  end

  def tohtml(date)
    body = super
    if date == @diaries[0] then
      body = "\n<!-- PiloWeb_Begin -->\n#{body}<!-- PiloWeb_End -->\n\n"
    end
    return body
  end

  def makeHTML
    @diaries = @allDiaries.keys.sort.reverse[0...@maxDiaries]
    lastDate = @diaries[0]

    @objLastDate = Time::local(lastDate[0..3], lastDate[4..5], lastDate[6..7])

    @filename = @latestFilename
    writeHTML
  end

end

class PastDiary < Diary

  attr_reader :targetMonth

  def initialize
    super()
    @targetMonth = ''
    @topics      = {}
  end

  def config=(conf)
    super
    @format = @config['DATEFORMAT_PASTDAYS'] if @config.key?('DATEFORMAT_PASTDAYS')
    @header = @config['HEADER_PASTDAYS'] if @config.key?('HEADER_PASTDAYS')
    @skelton = @config['SKELTON_PASTDAYS'] if @config.key?('SKELTON_PASTDAYS')
    @splithtml = @config['SPLIT_PASTDAYS'] if @config.key?('SPLIT_PASTDAYS')
    @tag_date  = @config['TAG_DATE_PASTDAYS'] if @config.key?('TAG_DATE_PASTDAYS')
    @tag_topic = @config['TAG_TOPIC_PASTDAYS'] if @config.key?('TAG_TOPIC_PASTDAYS')
  end

  def targetMonth=(month)
    @topics.clear
    @targetMonth = month
  end

  def makeHTML
    diaries = @allDiaries.keys.sort
    diaries.delete_if{ |file|
      file.index(@targetMonth) == nil
    }

    if diaries.empty? then
      $stderr.print @targetMonth + ": not found  .diary files\n"
      return
    end
    @objTargetMonth = Time::local(@targetMonth[0..3], @targetMonth[4..5])

    partdiaries = {}
    diaries.each{ |date|
      day = date[6..7].to_i
      unless partdiaries.key?(@monthPartNames[day]) then
        partdiaries[@monthPartNames[day]] = []
      end
      partdiaries[@monthPartNames[day]] << date
    }

    partdiaries.each{|part, ds|
      @diaries = ds.sort
      @filename = date2monthlyfilename(@diaries[0])
      writeHTML
    }

    # Topics List
    open(@logDirectory + @targetMonth + '.topic', "w"){ |file|
      @topics.keys.sort.each{ |dd|
        file.puts "#@targetMonth#{dd}\t#{@topics[dd].join(\"\t\")}"
      }
    } unless @topics.empty?
  end
end

# typo fix
class Config
  alias set_original set

  def set(key, val)
    key = case key
    when 'LASTEST_FILENAME'
      'LATEST_FILENAME'
    when 'SKELTON_LASTEST'
      'SKELTON_LATEST'
    when 'SPLIT_LASTEST'
      'SPLIT_LATEST'
    when 'DATEFORMAT_LASTEST'
      'DATEFORMAT_LATEST'
    when 'HEADER_LASTEST'
      'HEADER_LATEST'
    when 'TAG_DATE_LASTEST'
      'TAG_DATE_LATEST'
    when 'TAG_TOPIC_LASTEST'
      'TAG_TOPIC_LATEST'
    else
      key
    end
    set_original(key, val)
  end
end


#///////////////////////////////////////////////////////////////////
#
# Main Routine                                                      
#

def main

  ti = Time::now
  print "\n-- #{NDIARY_VERSION} --\n\n"

  if RUBY_VERSION < REQUIRED_RUBY_VER then
    $stderr.puts "\n** error: this script require ruby #{REQUIRED_RUBY_VER} or later.\n"
    exit
  end

  # Load Configuration
  if /.+\.conf$/i =~ ARGV[0] or /\.ndiaryrc$/ =~ ARGV[0] then
    configfile = ARGV.shift
  else
    configfile = ENV['HOME'].to_s + File::Separator + ".ndiaryrc";
    unless File::readable? configfile then
      configfile = 'ndiary.conf'
    end
  end

  unless File::readable? configfile then
    $stderr.puts "\n** error: config file '#{configfile}' not found.\n"
    exit
  end
  
  config = Config::new
  config.load(configfile)
  config.setarray('TOPIC', 'FILTER')

  filter = Filter::new(config['FILTER'], config['USER_FILTER_DIRECTORY'])
  config['MONTH_PART_NAMES'] = [] unless config['MONTH_PART_NAMES'].kind_of?(Array)

  while ARGV[0].to_s[0,1] == "-" do
    opt = ARGV.shift
    opt.sub!(/^\-/, '')
    key, value = opt.split('=', 2)
    if value.nil? then
      config[key] = true
    else
      begin
        config[key] = eval(value.tosjis)
      rescue
        config[key] = value.tosjis
      end
    end
  end

  config.freeze

  monthPartNames = [''] * 32
  monthPartNames[1,0] = config['MONTH_PART_NAMES']
  monthPartNames = monthPartNames[0..31]

  pastDiary                     = PastDiary::new
  pastDiary.config              = config
  pastDiary.monthPartNames      = monthPartNames
  pastDiary.filter              = filter
  pastDiary.getDiaries


   if pastDiary.allDiaries.keys.nil? or pastDiary.allDiaries.keys.size < 1 then
      $stderr.puts "\n** error: no .diary file under #{pastDiary.logDirectory}.\n"
      exit(1)
   end

  if ARGV.size.zero? then
    pastDiary.targetMonth = pastDiary.allDiaries.keys.sort.reverse[0][0..5]
    pastDiary.makeHTML

    lastDiary                     = LastDiary::new
    lastDiary.config              = config
    lastDiary.monthPartNames      = monthPartNames
    lastDiary.getDiaries(pastDiary)
    lastDiary.filter              = filter
    lastDiary.makeHTML
    year = lastDiary.allDiaries.keys.sort.reverse[0][0..3]

  elsif ARGV[0] =~ /^(\d{6})$/ then
    pastDiary.targetMonth = $1
    pastDiary.makeHTML
    year = ARGV[0][0..3]

  elsif ARGV[0].downcase == 'all' then
    for month in pastDiary.months
      pastDiary.targetMonth = month
      pastDiary.makeHTML
    end

  else
    $stderr.puts "\n** Bad Parameter: '#{ARGV[0]}'\n"
    exit
  end

  # make topic list
  if !config['TOPIC'].nil? and config['TOPIC'].size > 0 then
    t = Topics::new
    t.topicDateFormat  = config['TOPIC_DATE_FORMAT']
    t.topicMonthFormat = config['TOPIC_MONTH_FORMAT']
    t.diary            = pastDiary
    config['TOPIC'].each{ |topic|
      filename, regexp, pattern, replace = topic.split(/\t/)
      t.skelton = filename
      t.regexp  = regexp  ? regexp  : ''
      t.pattern = pattern ? pattern : ''
      t.replace = replace ? replace : ''
      t.writeHTML(pastDiary.allDiaries.keys, true, year)
    }
  end

  puts "\nfinish. (#{Time::now - ti}sec.)"
end
