#!/usr/local/bin/ruby
# data-read/write, html-table-write
# 2002.10.03 ver0.1 by ippo
# 2002.10.23 ver0.2 by ippo
# 2002.10.31 ver0.2a by ippo
# 2002.12.14 ver0.3 by ippo
# 2003.02.20 ver0.3a by ippo
# 2003.02.22 ver0.3b by ippo
# 2003.02.25 ver0.3c by ippo

class SFfile
  def ttcsv_reader ( file = nil, checkkey = nil, checktrack = nil )
#    $stderr.puts "CHK; file; " << file
    if file.nil? then fp = $stdin
    else              fp = File.open( file ) end
    text = []
    fp.each{ |line| 
      string = Kconv.toeuc(line).chomp.chomp("\r")
      text << string if string !~ /^#/
#      $stderr.puts string if string =~ /^#/
    }
    if !file.nil? then fp.close end
    key = text.shift.split("\t")
    data = []
    text.each_index{ |j|
      record = {}
      record["track"] = j if !checktrack.nil?
      info = text[j].split("\t")
      key.each_index { |i|
        str = info[i]
        str = '' if str.nil?
        record[key[i]] = str
      }
      data << record
    }
    return data      if  checkkey.nil?
    return data, key if !checkkey.nil?
  end

  def ttcsv_writer ( file, key, data )
    if file.nil? then fp = $stdout
    else              fp = File.open( file , "w") end
    fp.puts key.join("\t")
    data.each{ |line|
      out = []
      key.each{ |val| out << line[val] }
      fp.puts out.join("\t")
    }
    if !file.nil? then fp.close end
  end

  def log_reader ( file )
    if file.nil? then fp = $stdin
    else              fp = File.open( file ) end
    text = []
    fp.each{ |line| 
      string = Kconv.toeuc(line).chomp.chomp("\r")
      text << string if string !~ /^#/
#      $stderr.puts string if string =~ /^#/
    }
    if !file.nil? then fp.close end
    data = []
    text.each{ |line|
      data << line.split("\t")
    }
    return data
  end

  def log_writer ( file, data )
    if file.nil? then fp = $stdout
    else              fp = File.open( file , "a") end
    data.each{ |line|
      fp.puts line.join("\t")
    }
    if !file.nil? then fp.close end
  end

  def hash_reader ( file = nil, checkkey = nil )
    if file.nil? then fp = $stdin
    else              fp = File.open( file ) end
    text = []
    fp.each{ |line| 
      string = Kconv.toeuc(line).chomp.chomp("\r")
      text << string if string !~ /^#/
    }
    if !file.nil? then fp.close end
    key, data = [], {}
    text.each{ |line|
      line = line.split("\t")
      key << line[0]
      data[line[0]] = line[1]
    }
    return data      if  checkkey.nil?
    return data, key if !checkkey.nil?
  end

  def hash_writer ( file, data, key = nil )
    if file.nil? then fp = $stdout
    else              fp = File.open( file , "w") end
    key = data.keys if key.nil?
    key.each{ |name|
      fp.puts [name, data[name]].join("\t")
    }
    if !file.nil? then fp.close end
  end

  def table_writer (fp, data, step = 0)
    step = data.size if step == 0
    size = data.size
    str = ''
    0.step(size-1, step){ |i|
      j =  i + step - 1
      j = -1 if j >= size
      str << '<table border="1">' + "\n"
      data[i..j].each{ |line|
        str << '<tr><td>' + line.join('</td><td>') + '</td></tr>' + "\n"
      }
      str <<  '</table>' + "\n"
    }
    if fp.type == (IO||File)
      fp.print str
    end
    return str
  end

  def htable_writer (fp, data, step = 0, option='border="1"')
    step = data.size + 1 if step == 0
    size = data.size + 1
    data << data[0]
    str = ''
    0.step(size-1, step){ |i|
      j =  i + step - 1
      j = -1 if j >= size
      str << '<table ' + option + '>' + "\n"
      data[i..j].each_index{ |k|
        line = data[k]
        if k == 0 || k == size-1
          str << '<tr><th nowrap>' + line.join('</th><th nowrap>') + '</th></tr>' + "\n"
        else
          str << '<tr><td>' + line.join('</td><td>') + '</td></tr>' + "\n"
        end
      }
      str <<  '</table>' + "\n"
    }
    if fp.type == (IO||File)
      fp.print str
    end
    return str
  end

  def hzebratable_writer (fp, data, step = 0, option='bgcolor="#cccccc"')
    step = data.size + 1 if step == 0
    size = data.size + 1
    data << data[0]
    str = ''
    flag = 0
    0.step(size-1, step){ |i|
      j =  i + step - 1
      j = -1 if j >= size
      str << '<table ' + option + '>' + "\n"
      data[i..j].each_index{ |k|
        line = data[k]
        rcolor = (flag == 0) ? '#ddddff' : '#ffeeee'
        flag = 1 - flag
        if k == 0 || k == size-1
          str << '<tr bgcolor="' + rcolor + '"><th nowrap>' + line.join('</th><th nowrap>') + '</th></tr>' + "\n"
        else
          str << '<tr bgcolor="' + rcolor + '"><td>' + line.join('</td><td>') + '</td></tr>' + "\n"
        end
      }
      str <<  '</table>' + "\n"
    }
    if fp.type == (IO||File)
      fp.print str
    end
    return str
  end

  def zebratable_writer (fp, data, step = 0, option='bgcolor="#cccccc"', option2 = '')
    step = data.size if step == 0
    size = data.size
    str = ''
    flag = 0
    0.step(size-1, step){ |i|
      j =  i + step - 1
      j = -1 if j >= size
      str << '<table ' + option + '>' + "\n"
      data[i..j].each{ |line|
        rcolor = (flag == 0) ? '#ddddff' : '#ffeeee'
        flag = 1 - flag
        str << '<tr bgcolor="' + rcolor + '"><td' + option2 + '>' + line.join('</td><td>') + '</td></tr>' + "\n"
      }
      str << '</table>' + "\n"
    }
    if fp.type == (IO||File)
      fp.print str
    end
    return str
  end

  def zebracolcolortable_writer (fp, data, step = 0, colcolor = [])
    step = data.size if step == 0
    size = data.size
    str = ''
    flag = 0
    0.step(size-1, step){ |i|
      j =  i + step - 1
      j = -1 if j >= size
      str << '<table border="1">' + "\n"
      data[i..j].each{ |line|
        if flag == 0
          colred, colgreen, colblue = 255, 255, 255
        else
          colred, colgreen, colblue = 12 * 16 + 12, 12 * 16 + 12, 12 * 16 + 12
        end
        str << '<tr>'
        line.each_index{ |i|
          block = line[i]
          color = colcolor[i].nil? ? '000000' : colcolor[i]
          tmp = []
          0.upto(color.size-1){ |j|
            case color[j]
            when 'f'
              tmp << 15
            when 'e'
              tmp << 14
            when 'd'
              tmp << 13
            when 'c'
              tmp << 12
            when 'b'
              tmp << 11
            when 'a'
              tmp << 10
            else
              tmp << color[j..j].to_i
            end
          }
          red, green, blue = tmp[0] * 16 + tmp[1], tmp[2] * 16 + tmp[3], tmp[4] * 16 + tmp[5]
          red   = colred   - red
          green = colgreen - green
          blue  = colblue  - blue
          red   = 0   if red   < 0
          green = 0   if green < 0
          blue  = 0   if blue  < 0
          red   = 255 if red   > 255
          green = 255 if green > 255
          blue  = 255 if blue  > 255
          red   = "%02x" % red.to_i
          green = "%02x" % green.to_i
          blue  = "%02x" % blue.to_i
          str << '<td bgcolor="#' + red + green + blue + '">'
          str << block if block.size > 0
          str << '</td>'
        }
        str << '</tr>' + "\n"
        flag = 1 - flag
      }
      str << '</table>' + "\n"
    }
    if fp.type == (IO||File)
      fp.print str
    end
    return str
  end

  def gradetable_writer (fp, data, step = 0)
    per = 255.0 / (2.0 * data.size.to_f)
    step = data.size if step == 0
    size = data.size
    str = ''
    count = 0
    0.step(size, step){ |i|
      j =  i + step - 1
      j = -1 if j >= size
      str << '<table border="1">' + "\n"
      data[i..j].each{ |line|
        blue = "%02x" % (255 - (count.to_f * per).to_i)
        str << '<tr bgcolor="#ffff' + blue + '"><td>' + line.join('</td><td>') + '</td></tr>' + "\n"
        count += 1
      }
      str << '</table>' + "\n"
    }
    if fp.type == (IO||File)
      fp.print str
    end
    return str
  end
end
